#!/usr/bin/perl

BEGIN {
	require "fatlib.pl";
};

use utf8;
use strict;
use CGI;
use Encode qw/decode decode_utf8/;
use Encode::Guess;
use PlannedBlackoutJP::Util qw/is_galapagos/;
use Text::MicroTemplate::File;
use Nana::Cache;
use HTTP::Lite;
require "common.pl";

my $tokyodenryoku_base_url="http://www.tepco.co.jp/forecast/html/images/juyo-j.csv";
my $cachefile="powerusage";
my $debug;

	
&main;

sub main {
	my $query=new CGI;
	my $mode=$query->param('m');

	if($mode eq 'qr') {
		my $qr=$query->param('qr');
		my $string=$query->param('str');
		&make_qrcode($string,$qr);
		exit;
	}

	my $mobileflg;
	my $mflg=0;
	$mflg=1 if($mode=~/m/);
	my $englishflg=0;
	$englishflg=1 if($mode=~/e/);
	$mobileflg=1 if(is_galapagos(\%ENV) || $mflg);

	my $cache=new Nana::Cache (
		ext=>"csv",
		files=>100,
		dir=>"./cache",
		size=>100000,
		use=>1,
		expire=>30*60,
	);

	my $http_body=$cache->read($cachefile,0);
	if($http_body eq '' || $mode eq  'update') {
		my $status;
		($status,$http_body)=&httpcl($tokyodenryoku_base_url);
		if($status eq 0) {
			$cache->write($cachefile,$http_body);
		}
	}

	$http_body=~s/\x0D\x0A|\x0D|\x0A/\x0A/g;
	$http_body=&force_decode($http_body);

	my @pwtmp=split(/\n/,$http_body);
	my $update=$pwtmp[0];
	my $kyoukyuu=$pwtmp[2];
	my $maxpower=$pwtmp[5];
	my @pw;
	for(my $i=0; $i<24; $i++) {
		$pw[$i]=$pwtmp[$i+8];
	}
		
	my($update_date,$update_time,$tmp)=split(/ /,$update);
	my $update_str="$update_date $update_time";
	my($kyoukyuu_kw,$kyoukyuu_time,$kyoukyuu_date,$kyoukyuu_update_time)
		=split(/,/,$kyoukyuu);
	my($maxpower_kw,$maxpower_time,$maxpower_date,$maxpower_update_time)
		=split(/,/,$maxpower);
	my $today_persent=int(($maxpower_kw+0) / ($kyoukyuu_kw+0)*100);
	my $maxupdate="$maxpower_date $maxpower_update_time";

	my @hour_refs;
	for(my $i=0; $i<24; $i++) {
		my($pw_date,$pw_time,$today_power,$yesterday_power)
			=split(/,/,$pw[$i]);
		my $persent=int(($today_power+0) / ($kyoukyuu_kw+0)*100);
		my $diff=$today_power eq 0 ? 0 : $today_power - $yesterday_power;
		push @hour_refs,  {
			date => $pw_date,
			time => $pw_time,
			today_power => $today_power,
			yesterday_power => $yesterday_power,
			persent => $persent,
			diff => $diff,
		};
	}
	&gzip_compress("Content-type: text/html; charset=utf-8\nCache-Control: max-age=0\nExpires: Mon, 26, Jul 1997 05:00:00 GMT");
	my $mtf = Text::MicroTemplate::File->new;
	my $template;
	if($mflg eq 1) {
		if($englishflg eq 1) {
			$template="powerusage_en_m.html";
		} else {
			$template="powerusage_jp_m.html";
		}
	} else {
		if($englishflg eq 1) {
			$template="powerusage_en.html";
		} else {
			$template="powerusage_jp.html";
		}
	}
	my($basehref, $basehost, $basepath)=&getbasehref;
	my $qrmode= $englishflg ? "me" : "mj";
	my $html = $mtf->render_file(
		$template,
		"東京電力","TEPCO's",
		"$update_date $update_time",
		$kyoukyuu_kw,$kyoukyuu_time,
		$maxpower_kw,$maxpower_time,
		$today_persent,$maxupdate,
		\@hour_refs,
		&make_link_qrcode("$basehref?m=$qrmode")
);

	if($mobileflg) {
		$html=&z2h($html);
	}

	print <<EOM;
<?xml version="1.0" encoding="UTF-8" ?>
$html$debug
EOM
	}

sub httpcl {
	my ($url)=shift;
	my $http=new HTTP::Lite;
	$http->method("GET");
	$http->http11_mode(1);
	my $req=$http->request($url);
	if($req eq 200) {
		return(0,$http->body());
	} else {
		return(1,"Error $req");
	}
}

# QRコードの画像のリンクを作成
sub make_link_qrcode {
	my ($string) = shift;
	if(&load_module("GD") && &load_module("GD::Barcode")) {
		my($basehref, $basehost, $basepath)=&getbasehref;
		$string=&enc($string);
		return "$basehref?m=qr\&amp;str=$string";
	}
	'';
}
